/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef GAME_H
#define GAME_H

#include <string>
#include <vector>
#include <climits>

#include "defines.h"

using std::string;
using std::vector;

namespace Spellathon
{
	class Dict;

	class Game {
	public:
		Game(Dict *dct, bool plural_sieve);
		~Game();

		bool play(bool precomp, int minsol, int maxsol); 
		bool solve(string letters);
		string sig() const  { if (found_) return sig_; else return ""; }
		const vector<string>& words();
		static int sidxcmp(const void *a, const void *b);
		void setPluralSieve(bool e); 
		bool pluralSieve() const { return plural_sieve_; }
		int	solve(const char *letrs, int min = 0, int max = INT_MAX);
		int nwords() const { return wordPtrs.size(); }

	private:
		bool _play(int minsol, int maxsol);
		bool _play_precomp(int minsol, int maxsol);
		int	signGen(const char *letters, char signarr[][MAX_LETRS+1]);
	private:
		Dict	*dict_;
		string	sig_;
		bool	found_;
		bool plural_sieve_;
		vector<string> words_;
		vector<char*>	wordPtrs;
		Game(const Game &);
		Game & operator=(const Game &);
	};
}

#endif /* GAME_H */
